# -*- encoding : utf-8 -*-

require 'base64'
require 'openssl'

require_relative 'version'
require_relative 'protobuf'

module AliyunLogRubySdk
  module Headers
    X_LOG_SIGNATURE_METHOD = 'hmac-sha1'.freeze

    def generate_headers(options = {})
      headers = options[:headers] || {}
      host = headers.delete('x-log-host')

      headers = headers.merge('x-log-apiversion' => X_LOG_API_VERSION,
                              'x-log-signaturemethod' => X_LOG_SIGNATURE_METHOD,
                              'Date' => Time.now.httpdate,
                              'Host' => (host || "#{project_name}.#{endpoint}"))

      body = options[:body]

      if body.present?
        headers['Content-Length'] = body.bytesize.to_s
        headers['Content-MD5'] = calculate_md5(body)
      else
        headers['Content-Length'] = '0'
        headers['x-log-bodyrawsize'] = '0'
      end

      headers['Authorization'] = generate_authorization(options[:verb] || 'GET', headers, options[:resource])

      headers['x-log-date'] = headers['Date']

      headers
    end

    private

    def generate_authorization(verb, headers, resource)
      "LOG #{access_key_id}:#{generate_signature(verb, headers, resource)}"
    end

    def generate_signature(verb, headers, resource)
      Base64.encode64(OpenSSL::HMAC.digest('sha1', access_key_secret, generate_sign_string(verb, headers, resource).encode('utf-8'))).rstrip
    end

    def generate_sign_string(verb, headers, resource)
      %W[
        #{verb}
        #{headers['Content-MD5']}
        #{headers['Content-Type']}
        #{headers['Date']}
        #{canonicalized_log_headers(headers)}
        #{resource}
      ].join("\n")
    end

    def calculate_md5(body)
      Digest::MD5.hexdigest(body).upcase
    end

    def canonicalized_log_headers(headers)
      headers.select { |key, _value| key.match(/x-log-*|x-acs-*/) }.map do |key, value|
        "#{key.downcase}:#{value}"
      end.sort.join("\n")
    end
  end
end
